﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNet.Builder;
using Microsoft.AspNet.Hosting;
using Microsoft.AspNet.Http;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Configuration;
using ASPNETCORE1.Services;
using Microsoft.AspNet.Routing;
using ASPNETCORE1.Entities;
using Microsoft.Data.Entity;
using Microsoft.AspNet.Identity.EntityFramework;

namespace ASPNETCORE1
{
    public class Startup
    {

        public IConfiguration Configuration { get; set; }
        public Startup()
        {
            var builder = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json");
            //
            Configuration = builder.Build();
        }

        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit http://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddMvc();
            //
            services.AddEntityFramework()
                .AddSqlServer()
                .AddDbContext<RestaurantManagementDbContext>(options => options.UseSqlServer(Configuration["database:connection"]));
            //
            services.AddIdentity<User, IdentityRole>()
                .AddEntityFrameworkStores<RestaurantManagementDbContext>();
            //
            services.AddSingleton(provider => Configuration);
            services.AddSingleton<IGreeter, Greeter>();
            services.AddScoped<IRestauranData, SqlRestaurantData>();
        }

        // This method gets called by the runtime. 
        //Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app,
            IGreeter greeter,
            IHostingEnvironment environment)
        {
            app.UseIISPlatformHandler();
            //app.UseWelcomePage();
            if (environment.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            //
            app.UseFileServer();
            app.UseRuntimeInfoPage("/inf");
            //
            //app.UseDefaultFiles();
            //app.UseStaticFiles();

            //app.UseMvcWithDefaultRoute();
            app.UseIdentity();

            app.UseMvc(ConfigureRoutes);
           
            app.Run(async (context) =>
            {
               // throw new Exception("Error!!.");
                var greeting = greeter.GetGreeter();
                await context.Response.WriteAsync(greeting);
                
            });
        }

        private void ConfigureRoutes(IRouteBuilder routeBuilder)
        {
            //Home/Index
            routeBuilder.MapRoute("Default","{Controller=Home}/{Action=Index}/{id?}");
        }

        // Entry point for the application.
        public static void Main(string[] args) => WebApplication.Run<Startup>(args);
    }
}
